package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.BasicAgreement;
import cn.com.infosec.crypto.BufferedBlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DerivationFunction;
import cn.com.infosec.crypto.InvalidCipherTextException;
import cn.com.infosec.crypto.Mac;
import cn.com.infosec.crypto.params.IESParameters;
import cn.com.infosec.crypto.params.IESWithCipherParameters;
import cn.com.infosec.crypto.params.KDFParameters;
import cn.com.infosec.crypto.params.KeyParameter;
import java.math.BigInteger;

public class IESEngine {
   BasicAgreement agree;
   DerivationFunction kdf;
   Mac mac;
   BufferedBlockCipher cipher;
   byte[] macBuf;
   boolean forEncryption;
   CipherParameters privParam;
   CipherParameters pubParam;
   IESParameters param;

   public IESEngine(BasicAgreement agree, DerivationFunction kdf, Mac mac) {
      this.agree = agree;
      this.kdf = kdf;
      this.mac = mac;
      this.macBuf = new byte[mac.getMacSize()];
      this.cipher = null;
   }

   public IESEngine(BasicAgreement agree, DerivationFunction kdf, Mac mac, BufferedBlockCipher cipher) {
      this.agree = agree;
      this.kdf = kdf;
      this.mac = mac;
      this.macBuf = new byte[mac.getMacSize()];
      this.cipher = cipher;
   }

   private byte[] decryptBlock(byte[] in_enc, int inOff, int inLen, byte[] z) {
      KDFParameters inLen1;
      inLen1 = new KDFParameters.<init>(z, this.param.getDerivationV());
      int z1 = this.param.getMacKeySize();
      this.kdf.init(inLen1);
      int var5 = inLen - this.mac.getMacSize();
      byte[] var7;
      byte[] var8;
      KeyParameter var14;
      if (this.cipher == null) {
         int var6;
         var7 = this.generateKdfBytes(inLen1, var5 + (var6 = z1 / 8));
         var8 = new byte[var5];

         for(inLen = 0; inLen != var5; ++inLen) {
            var8[inLen] = (byte)(in_enc[inOff + inLen] ^ var7[inLen]);
         }

         KeyParameter inLen2;
         inLen2 = new KeyParameter.<init>(var7, var5, var6);
         var14 = inLen2;
      } else {
         byte[] var13 = this.generateKdfBytes(inLen1, (inLen = ((IESWithCipherParameters)this.param).getCipherKeySize() / 8) + (z1 /= 8));
         BufferedBlockCipher var17 = this.cipher;
         var14 = new KeyParameter.<init>(var13, 0, inLen);
         var17.init(false, var14);
         byte[] var10000 = var7 = new byte[this.cipher.getOutputSize(var5)];
         int var15;
         int var16;
         var8 = new byte[var16 = (var15 = this.cipher.processBytes(in_enc, inOff, var5, var7, 0)) + this.cipher.doFinal(var7, var15)];
         System.arraycopy(var10000, 0, var8, 0, var16);
         var14 = new KeyParameter.<init>(var13, inLen, z1);
      }

      byte[] inLen3 = this.param.getEncodingV();
      this.mac.init(var14);
      this.mac.update(in_enc, inOff, var5);
      this.mac.update(inLen3, 0, inLen3.length);
      this.mac.doFinal(this.macBuf, 0);
      inOff += var5;

      for(inLen = 0; inLen < (z = this.macBuf).length; ++inLen) {
         if (z[inLen] != in_enc[inOff + inLen]) {
            throw new InvalidCipherTextException("Mac codes failed to equal.");
         }
      }

      return var8;
   }

   private byte[] encryptBlock(byte[] in, int inOff, int inLen, byte[] z) {
      KDFParameters var5;
      var5 = new KDFParameters.<init>(z, this.param.getDerivationV());
      int z1 = this.param.getMacKeySize();
      byte[] var7;
      byte[] var8;
      KeyParameter in1;
      if (this.cipher == null) {
         int var6;
         var7 = this.generateKdfBytes(var5, inLen + (var6 = z1 / 8));
         var8 = new byte[inLen + this.mac.getMacSize()];

         for(z1 = 0; z1 != inLen; ++z1) {
            var8[z1] = (byte)(in[inOff + z1] ^ var7[z1]);
         }

         in1 = new KeyParameter.<init>(var7, inLen, var6);
      } else {
         int var13;
         byte[] var14 = this.generateKdfBytes(var5, (var13 = ((IESWithCipherParameters)this.param).getCipherKeySize() / 8) + (z1 /= 8));
         BufferedBlockCipher var10001 = this.cipher;
         KeyParameter var15;
         var15 = new KeyParameter.<init>(var14, 0, var13);
         var10001.init(true, var15);
         byte[] var10000 = var7 = new byte[this.cipher.getOutputSize(inLen)];
         int in2;
         var8 = new byte[(inLen = (in2 = this.cipher.processBytes(in, inOff, inLen, var7, 0)) + this.cipher.doFinal(var7, in2)) + this.mac.getMacSize()];
         System.arraycopy(var10000, 0, var8, 0, inLen);
         in1 = new KeyParameter.<init>(var14, var13, z1);
      }

      byte[] var9 = this.param.getEncodingV();
      this.mac.init(in1);
      this.mac.update(var8, 0, inLen);
      this.mac.update(var9, 0, var9.length);
      this.mac.doFinal(var8, inLen);
      return var8;
   }

   private byte[] generateKdfBytes(KDFParameters kParam, int length) {
      byte[] var10000 = new byte[length];
      this.kdf.init(kParam);
      this.kdf.generateBytes(var10000, 0, length);
      return var10000;
   }

   public void init(boolean forEncryption, CipherParameters privParam, CipherParameters pubParam, CipherParameters param) {
      this.forEncryption = forEncryption;
      this.privParam = privParam;
      this.pubParam = pubParam;
      this.param = (IESParameters)param;
   }

   public byte[] processBlock(byte[] in, int inOff, int inLen) {
      this.agree.init(this.privParam);
      BigInteger var4 = this.agree.calculateAgreement(this.pubParam);
      IESEngine var10000;
      byte[] var5;
      if (this.forEncryption) {
         var10000 = this;
         var5 = var4.toByteArray();
         return var10000.encryptBlock(in, inOff, inLen, var5);
      } else {
         var10000 = this;
         var5 = var4.toByteArray();
         return var10000.decryptBlock(in, inOff, inLen, var5);
      }
   }
}
